#include <stdio.h>
#include <DS3231.h>
#include <toneAC.h>
#include <TM1638.h>
#include <EEPROM.h>

const int tempPin = A0;

const int clock0 = 27;
const int data = 29;

const int strobe = 31;
const int strobe2  = 33;
const int strobe3 = 35;

int dots = 0x0;

TM1638 module1(data, clock0, strobe);
TM1638 module2(data, clock0, strobe2);
TM1638 module3(data, clock0, strobe3);

const int SDA1 = 52; 
const int SCL1 = 50; 

DS3231  rtc(SDA1, SCL1);

int mon[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

Time t1;

int alarmVol = 0;
int alarmTone = 0;
int alarmPause = 0; 

int alarmMelody[] = { 262, 330, 392, 262, 330, 392, 0 };
int alarmCount = 0;


int sec1 = 0;
int sec10 = 0;
int min1 = 0;
int min10 = 0;
int hour1 = 0;
int hour10 = 0;
int year1 = 0;
int year10 = 0;
int mon1 = 0;
int mon10 = 0;
int day1 = 0;
int day10 = 0;
int dow = 0;

//
// 7 alarms
//

int amin1[7];
int amin10[7];
int ahour1[7];
int ahour10[7];

byte alarmSet = 0;
int settingAlarm = 0;
int currentAlarm = 0;
int lastAlarm = 0;

boolean blinkDate = true;
boolean blinkTime = true;
boolean ledBlink = true;
boolean blinkAlarm = true;

int blinkPos = -1;
int lastBlink = -1;

int mode = 0;
int mode1 = 0;

int buttonCount = 0;
int lastButtons = 0;

int month(int a, int yy)
{
  int x = 0, c;
  for (c = 0; c < a - 1; c++)
  {
    if (c == 1)
    {
      if (yy % 4 == 0)
        x += 29;
      else
        x += 28;
    }
    else
      x += mon[c];
  }
  return (x);
}

int days(int y1, int y2, int m1, int m2, int d1, int d2) {

  int count = 0;

  for (int i = y1; i < y2; i++)
  {
    if (i % 4 == 0)
      count += 366;
    else
      count += 365;
  }
  count -= month(m1, y1);
  count -= d1;
  count += month(m2, y2);
  count += d2;

  return count;
}

void pciSetup(byte pin)
{
  *digitalPinToPCMSK(pin) |= bit (digitalPinToPCMSKbit(pin));  // enable pin
  PCIFR  |= bit (digitalPinToPCICRbit(pin)); // clear any outstanding interrupt
  PCICR  |= bit (digitalPinToPCICRbit(pin)); // enable interrupt for the group
}

void setup() {

  analogReference(EXTERNAL);

  module1.clearDisplay();
  module2.clearDisplay();
  module3.clearDisplay();

  rtc.begin();
  t1 = rtc.getTime();

  //Serial.begin(9600);
  pinMode(10, INPUT);
  pciSetup(10);

  int address = 0;

  if (module3.getButtons()) {
    for (int i = 0 ; i < EEPROM.length() ; i++) {
      EEPROM.write(i, 0);
      module3.setLEDs(i);
    }
  }


  address = 0;
  for (int i = 0; i < 7; i++) {
    amin1[i]   = EEPROM.read(address++);
    amin10[i]  = EEPROM.read(address++);
    ahour1[i]  = EEPROM.read(address++);
    ahour10[i] = EEPROM.read(address++);
  }

}

void storeAlarms() {
  int address = 0;
  for (int i = 0; i < 7; i++) {
    EEPROM.write(address++, amin1[i]);
    EEPROM.write(address++, amin10[i]);
    EEPROM.write(address++, ahour1[i]);
    EEPROM.write(address++, ahour10[i]);
    //module3.setLEDs(i);

  }
}


ISR (PCINT0_vect) // handle pin change interrupt for D0 to D7 here
{ // don't trigger while alarm is playing, alarm melody needs to have pauses!
  if (alarmPause > 0)
    alarmOff();
}

void setCurTime() {
  rtc.setTime(hour10 * 10 + hour1, min10 * 10 + min1, sec10 * 10 + sec1);
}

void setCurDate() {
  rtc.setDate(day10 * 10 + day1, mon10 * 10 + mon1, year10 * 10 + year1 + 2000);
  rtc.setDOW(dow);
}


void showTime() {

  if (blinkTime) {

    module1.sendChar(7, NUMBER_FONT[sec1], false);
    module1.sendChar(6, NUMBER_FONT[sec10], false);

    module1.sendChar(4, NUMBER_FONT[min1], false);
    module1.sendChar(3, NUMBER_FONT[min10], false);

    module1.sendChar(1, NUMBER_FONT[hour1], false);
    module1.sendChar(0, NUMBER_FONT[hour10], false);

  } else

    if (blinkPos == 8)
      module1.clearDisplay();
    else if (blinkPos >= 0)
      module1.sendChar(blinkPos, 0, false);

}

void showDate() {

  if (blinkDate) {

    module2.sendChar(1, NUMBER_FONT[mon1], true);
    module2.sendChar(0, NUMBER_FONT[mon10], false);

    module2.sendChar(3, NUMBER_FONT[day1], true);
    module2.sendChar(2, NUMBER_FONT[day10], false);

    module2.sendChar(5, NUMBER_FONT[year1], false);
    module2.sendChar(4, NUMBER_FONT[year10], false);

    module2.sendChar(7, NUMBER_FONT[dow], false);

  } else

    if (blinkPos == 8)
      module1.clearDisplay();
    else if (blinkPos >= 0)
      module2.sendChar(blinkPos, 0, false);

}



void showAlarm(int alarm) {

  if (blinkAlarm) {

    module3.sendChar(0, FONT_DEFAULT['A'], false);
    module3.sendChar(1, FONT_DEFAULT['L'], false);
    module3.sendChar(2, NUMBER_FONT[alarm], false);
    module3.sendChar(3, 0, false);
    module3.sendChar(5, NUMBER_FONT[ahour1[alarm - 1]], true);
    module3.sendChar(4, NUMBER_FONT[ahour10[alarm - 1]], false);
    module3.sendChar(7, NUMBER_FONT[amin1[alarm - 1]], false);
    module3.sendChar(6, NUMBER_FONT[amin10[alarm - 1]], false);

  } else

    if (blinkPos == 8)
      module3.clearDisplay();
    else if (blinkPos >= 0)
      module3.sendChar(blinkPos, 0, false);

}

void showStatus() {

  if ( lastAlarm > 0 && alarmVol > 0) {

    showAlarm(lastAlarm);

  } else if (mode < 3) {

    unsigned long diff = days(2011, 2000 + year10 * 10 + year1 , 12, mon10 * 10 + mon1, 14, day10 * 10 + day1);

    int tempReading = analogRead(tempPin);
    int val = analogRead(tempPin);
    float mv = ( val / 1024.0) * 3300;
    float cel = mv / 10;
    float farh = (cel * 9) / 5 + 32;

    switch (mode1) {
      case 0 : diff = diff; break;
      case 1 : diff = diff * 24 + hour10 * 10 + hour1; break;
      case 2 : diff = diff * 24 * 60 + ( hour10 * 10 + hour1 ) * 60 + min10 * 10 + min1; break;
      case 3 : diff = farh; break;
      case 4 : diff = cel; break;
    }

    module3.setDisplayToDecNumber(diff, 0, false);

    if (mode1 == 3) {
      module3.sendChar(0, NUMBER_FONT[15], false);
      module3.sendChar(1, FONT_DEFAULT[10], false);
    } else if (mode1 == 4) {
      module3.sendChar(0, NUMBER_FONT[12], false);
      module3.sendChar(1, FONT_DEFAULT[10], false);
    }

  }  else

    showAlarm(settingAlarm);

  int leds = 0; 
  boolean alarmOn = false; 
  if (alarmVol > 0 || alarmPause > 0 ) {
    ledBlink = true; 
    leds = 1 << ( alarmCount + 1) << 8 | 0b11111111;
    alarmOn = true; 
  } else {
    ledBlink = !ledBlink;  
    leds = alarmSet << 1;
  }

  switch ( mode ) {
    case 0 : module3.setLEDs(alarmOn ? leds : 1 | leds); break;
    case 1 : module3.setLEDs(ledBlink ? 0b1101101100000000 : 0 ); break;
    case 2 : module3.setLEDs(ledBlink ? 0b1011111100000000 : 0 ); break;
    default : leds = ledBlink ? 0b11110000 : 0;
      leds =  ledBlink ? leds | (((mode - 2) << 1 << 8)) : leds;
      module3.setLEDs(leds);
      break;
  }
}

int decodeByte(int byte) {

  for (int i = 0; i < 8; i++) {
    if (byte & ( 1 <<  i))
      return i;
  }

  return -1;

}

int maxDaysPerMonth(int month) {

  switch (month) {
    case 1 : return 31;
    case 2 : return ( (year10 * 10 + year1) % 4 == 0  ? 29 : 28);
    case 3 : return 31;
    case 4 : return 30;
    case 5 : return 31;
    case 6 : return 30;
    case 7 : return 31;
    case 8 : return 31;
    case 9 : return 30;
    case 10 : return 31;
    case 11 : return 30;
    case 12 : return 31;
  }
  return 0;
}

int checkForAlarm() {

  for (int i = 0; i < 7; i++) {

    if (alarmSet & (1 << i )) {
      if (hour10 == ahour10[i] &&
          hour1 == ahour1[i] &&
          min10 == amin10[i] &&
          min1 == amin1[i] )
        return i + 1;
    }
  }

  return 0;

}

void playAlarm() {

  if (alarmVol > 0) {

    if (alarmPause > 0) {

      alarmPause++;
      if (alarmPause == 6) {
        alarmPause = 0;
      }
    } else {

      /* if (alarmCount % 2 == 0)
        toneAC(440, alarmVol, 150);
        else
        toneAC(440, 0, 150); */

      alarmTone  = alarmMelody[alarmCount];
      
      alarmCount++;

      if (alarmTone == 0 ) {
        toneAC(0, alarmVol, 1000);
        alarmPause = 1;
      } else {
        toneAC(alarmTone, alarmVol, 150);
        alarmPause = 0;
      }

      if (alarmCount == 7) {
        if (alarmVol < 10)
          alarmVol++;
        alarmCount = 0;
      }
    }
  }
}

void alarmOff() {
  alarmVol = 0;
  alarmPause = 0; 
}

void loop() {

  if (mode == 0) {

    t1 = rtc.getTime();

    sec1 = t1.sec % 10;
    sec10 = t1.sec / 10;

    min1 = t1.min % 10;
    min10 = t1.min / 10;

    hour1 = t1.hour % 10;
    hour10 = t1.hour / 10;

    dow = t1.dow;

    mon1 = t1.mon % 10;
    mon10 = t1.mon / 10;

    day1 = t1.date % 10;
    day10 = t1.date / 10;

    year1 = t1.year % 10;
    year10 = t1.year % 100 / 10;

  }

  //
  //
  //

  showTime();
  showDate();
  showStatus();

  //
  //
  //

  byte buttons = module3.getButtons();

  if (mode == 0) {

    if (alarmVol > 0 )  {

      // cancel alarm?
      if (buttons != 0) {
        alarmVol = 0;
      }

      blinkPos = 8;
      blinkTime = !blinkTime;
      blinkDate = !blinkDate;
      blinkAlarm = !blinkAlarm;

    }
    else {

      blinkDate = true;
      blinkTime = true;
      blinkAlarm = true;

    }
  } else if (mode == 1) {

    blinkDate = true;
    blinkAlarm = true;
    blinkTime = !blinkTime;


  } else if (mode == 2) {

    blinkDate = !blinkDate;
    blinkTime = true;
    blinkAlarm = true;

  } else if (mode > 2) {

    blinkAlarm = !blinkAlarm;
    blinkDate = true;
    blinkTime = true;

  }

  //
  //
  //

  if (buttons != 0 && buttonCount == 0) {
    blinkTime = true;
    blinkDate = true;
    blinkAlarm = true;
    ledBlink = true;
    showTime();
    showDate();
    showStatus();
  }

  //
  //
  //

  if (buttons == 0 && lastButtons == 1 && mode == 0) {
    // toggle between days / hours / minutes display time known, and temperature (not yet)
    mode1 += 1;
    if (mode1 == 5)
      mode1 = 0;
    // initiate test alarm
    // alarmVol = 1;
  }

  //
  // alarms
  //

  if (mode == 0) {

    currentAlarm = checkForAlarm();

    if (currentAlarm > 0 && lastAlarm != currentAlarm) {
      // alarm new and was not canceled yet?
      alarmVol = 1;      
      alarmCount = 0; 
      lastAlarm = currentAlarm;
    }

    if (currentAlarm == 0) {
      // cancel after 1 minute
      alarmOff();
      lastAlarm = 0; 
    }

    if (alarmVol > 0)
      playAlarm();
  }

  //
  //
  //

  if (buttons == lastButtons) {
    buttonCount++;
  } else
    buttonCount = 0;

  if (buttons == lastButtons && buttonCount > 5 ) {
    if (buttons == 1) {
      if (mode == 0) {
        mode = 1;
        buttonCount = 0;
      } else if (mode == 1) {
        mode = 2;
        buttonCount = 0;
      }
    } else if (buttons > 1 ) {
      if (mode == 0) {
        // alarmSet
        mode =  2 + ( buttons >> 1 ) ;
        buttonCount = 0;
        settingAlarm = decodeByte(lastButtons);
        blinkPos = 4;
        alarmVol = 0;
        lastAlarm = 0;
      }
    }
  }

  if (mode == 0 && lastButtons > 1 && buttons == 0) {
    alarmVol = 0;
    lastAlarm = 0;
    alarmSet ^= (lastButtons >> 1);
    int leds = alarmSet << 1;
    module3.setLEDs(leds );
    // clear:
    blinkAlarm = true;
    settingAlarm = decodeByte(lastButtons);
    showAlarm(settingAlarm);
    delay(1000);
  }

  if (buttons == lastButtons && buttons == 0 && buttonCount > 30) {
    mode = 0;
    blinkPos  = -1;
    if (settingAlarm > 0) {
      storeAlarms();
      settingAlarm = 0;
    }
    buttonCount = 0;
  }

  //
  //
  //

  if ((mode == 1 || mode == 2 || mode > 2) && buttonCount == 0)  {

    if (buttons != 0) {

      int newBlink = decodeByte(buttons);

      if (mode > 2 ) {
        if (newBlink > 3 ) {
          blinkPos = newBlink;
          if (lastBlink == blinkPos) {

            int index = settingAlarm - 1 ;
            int aahour10 = ahour10[index];
            int aahour1 = ahour1[index];
            int aamin10 = amin10[index];
            int aamin1 = amin1[index];

            switch ( blinkPos ) {
              case 4 : aahour10 = aahour10 == 2 ? 0 : aahour10 + 1; break;
              case 5 : aahour1 = aahour1 == 9 || aahour10 == 2 && aahour1 == 3 ? 0 : aahour1 + 1; break;
              case 6 : aamin10 = aamin10 == 5 ? 0 : aamin10 + 1; break;
              case 7 : aamin1 = aamin1 == 9 ? 0 : aamin1 + 1; break;
            }
            if (aahour10 * 10 + aahour1 > 23) {
              aahour10 = 2;
              aahour1 = 0;
            }
            ahour10[index] = aahour10;
            ahour1[index] = aahour1;
            amin10[index] = aamin10;
            amin1[index] = aamin1;

            // setCurTime();
            // to do - save the alarm when set!!!
          }
          lastBlink = blinkPos;

        }
      } else if (mode == 1) {
        if (newBlink != 2 && newBlink != 5) {
          blinkPos = newBlink;
          if (lastBlink == blinkPos) {
            switch ( blinkPos ) {
              case 0 : hour10 = hour10 == 2 ? 0 : hour10 + 1; break;
              case 1 : hour1 = hour1 == 9 || hour10 == 2 && hour1 == 3 ? 0 : hour1 + 1; break;
              case 3 : min10 = min10 == 5 ? 0 : min10 + 1; break;
              case 4 : min1 = min1 == 9 ? 0 : min1 + 1; break;
              case 6 : sec10 = sec10 == 5 ? 0 : sec10 + 1; break;
              case 7 : sec1 = sec1 == 9 ? 0 : sec1 + 1;
            }
            if (hour10 * 10 + hour1 > 23) {
              hour10 = 2;
              hour1 = 0;
            }
            setCurTime();
          }
          lastBlink = blinkPos;

        }
      } else if (mode == 2) {
        if (newBlink != 6) {
          blinkPos = newBlink;
          if (lastBlink == blinkPos) {
            switch ( blinkPos ) {
              case 0 : mon10 = mon10 == 1 ? 0 : mon10 + 1; break;
              case 1 : mon1 = mon10 == 0 ? ( mon1 == 9 ? 1 : mon1 + 1 ) : ( mon1 == 2 ? 0 : mon1 + 1) ; break;
              case 2 : day10 = ( maxDaysPerMonth(mon10 * 10 + mon1) / 10 ) == day10 ? 0 : day10 + 1; break;
              case 3 : day1 = ( maxDaysPerMonth(mon10 * 10 + mon1) == (day10 * 10 + day1)) || day1 == 9 ?
                                ( day10 == 0 ? 1 : 0 ) : day1 + 1; break;
              case 4 : year10 = year10 == 9 ? 0 : year10 + 1; break;
              case 5 : year1 = year1 == 9 ? 0 : year1 + 1; break;

              case 7 : dow = dow == 7 ? 1 : dow + 1;
            }

            if (day1 + 10 * day10 > maxDaysPerMonth(mon10 * 10 + mon1)) {
              day10 = 0;
              day1 = 1;
            }

            if (year10 * 10 + year1 < 17 ) {

              year1 = 7;
              year10 = 1;

            }

            setCurDate();
          }
          lastBlink = blinkPos;

        }
      }
    }
  }


  lastButtons = buttons;

  delay (150);

}


